/*
 * Copyright (C) 2007 by Rui Dias <ruijdias@users.sourceforge.net>
 *
 * This file contains code from the libdvdread project.
 *
 * libdvdread: Provides a simple foundation for reading DVD-Video images.
 * Copyright (C) 2001, 2002 Billy Biggs <vektor@dumbterm.net>,
 *                          Hkan Hjort <d95hjort@dtek.chalmers.se>,
 *                          Bjrn Englund <d4bjorn@dtek.chalmers.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Module definition
 */
#ifdef DOC
%define DOCSTRING "The DVD access interface.\n\n\
This module contains the functions that form the interface to \
reading files located on a DVD."
%enddef
%module(docstring=DOCSTRING) dvd_reader
%feature("autodoc", "0");
#endif


/**
 * Includes
 */
%{
#include <dvdread/dvd_reader.h>
%}
%include cstring.i


/**
 * The length of one Logical Block of a DVD.
 */
#define DVD_VIDEO_LB_LEN 2048


/**
 * Maximum length of filenames allowed in UDF.
 */
#define MAX_UDF_FILE_NAME_LEN 2048


/**
 * DVDVersion
 */
%feature("autodoc", "DVDVersion() -> int\n\n\
Returns the version of libdvdread.") DVDVersion;
int DVDVersion(void);


/**
 * DVDOpen
 */
%feature("autodoc", "DVDOpen(path) -> dvd\n\n\
Opens a block device of a DVD-ROM file, or an image file, or a directory \n\
name for a mounted DVD or HD copy of a DVD.\n\n\
If the given file is a block device, or is the mountpoint for a block \n\
device, then that device is used for CSS authentication using libdvdcss.\n\
If no device is available, then no CSS authentication is performed, \n\
and we hope that the image is decrypted.\n\n\
If the path given is a directory, then the files in that directory may be \n\
in any one of these formats:\n\n\
  path/VIDEO_TS/VTS_01_1.VOB\n\
  path/video_ts/vts_01_1.vob\n\
  path/VTS_01_1.VOB\n\
  path/vts_01_1.vob\n\n\
On failure, RuntimeError is raised.\n\n\
Threads: this function uses chdir() and getcwd().\n\
The current working directory is global to all threads, \n\
so using chdir/getcwd in another thread could give unexpected results.") DVDOpen;
%exception DVDOpen {
    $action
    if (!result) {
        PyErr_SetString(PyExc_RuntimeError, "Unable to open DVD");
     return NULL;
  }
}
dvd_reader_t *DVDOpen( const char * );


/**
 * DVDClose
 */
%feature("autodoc", "DVDClose(dvd)\n\n\
Closes and cleans up the DVD reader object.\n\n\
NOTE: You must close all open files before calling this function.") DVDClose;
void DVDClose( dvd_reader_t * );


/**
 * DVDInit
 */
%feature("autodoc", "DVDInit()\n\n\
Initializes support for multiple threads.\n\n\
NOTE: You must call DVDInit before using any of the other functions\n\
if you are going to use them in a multithreading app.") DVDInit;
void DVDInit(void);


/**
 * DVDFinish
 */
%feature("autodoc", "DVDFinish()\n\n\
Frees any dlopened objects.\n\n\
NOTE: You must DVDClose all handles opened with DVDOpen before calling this.") DVDFinish;
void DVDFinish(void);


/**
 * File Domains
 */
typedef enum {
  DVD_READ_INFO_FILE,        /**< VIDEO_TS.IFO  or VTS_XX_0.IFO (title) */
  DVD_READ_INFO_BACKUP_FILE, /**< VIDEO_TS.BUP  or VTS_XX_0.BUP (title) */
  DVD_READ_MENU_VOBS,        /**< VIDEO_TS.VOB  or VTS_XX_0.VOB (title) */
  DVD_READ_TITLE_VOBS        /**< VTS_XX_[1-9].VOB (title).  All files in 
                                the title set are opened and read as a
                                single file. */
} dvd_read_domain_t;


/**
 * dvd_stat_t
 */
%feature("autodoc", "\
Contains information about the size of a file and the number of parts\n\
in case of a multipart file and the respective sizes of the parts.\n\n\
A multipart file is for instance VTS_02_1.VOB, VTS_02_2.VOB...\n\
The size of VTS_02_1.VOB will be stored in parts_size[0],\n\
VTS_02_2.VOB in parts_size[1]...\n\n\
The total size (sum of all parts) is stored in size and nr_parts will\n\
hold the number of parts.\n\n\
Only DVD_READ_TITLE_VOBS (VTS_??_[1-9].VOB) can be multipart files.") dvd_stat_t;
typedef struct {
  off_t size;          /**< Total size of file in bytes */
  int nr_parts;           /**< Number of file parts */
  off_t parts_size[9]; /**< Size of each part in bytes */
} dvd_stat_t;
%extend dvd_stat_t {
    void set_parts_size(int index, off_t obj)
    {
        $self->parts_size[index] = obj;
    }
};


/**
 * DVDFileStat
 */
%feature("autodoc", "DVDFileStat(dvd, titlenum, domain) -> dvd_stat_t\n\n\
Stats a file on the DVD given the title number and domain.\n\n\
This function is only of use if you want to get the size of each file\n\
in the filesystem. These sizes are not needed to use any other functions.\n\n\
RuntimeError is raised on failure.") DVDFileStat_aux;
%exception DVDFileStat_aux {
    $action
    if (!result) {
        PyErr_SetString(PyExc_RuntimeError, "Unable to stat file");
     return NULL;
  }
}
%rename(DVDFileStat) DVDFileStat_aux;
%newobject DVDFileStat;
%inline %{
dvd_stat_t *DVDFileStat_aux(dvd_reader_t *dvd, int titlenum, dvd_read_domain_t domain) {
    dvd_stat_t *stat = (dvd_stat_t *) malloc(sizeof(dvd_stat_t));
    int ret = DVDFileStat(dvd, titlenum, domain, stat);
    if (ret == -1)
    {
        free(stat);
        return NULL;
    }
    return stat;
}
%}


/**
 * DVDOpenFile
 */
%feature("autodoc", "DVDOpenFile(dvd, titlenum, domain) -> dvd_file\n\n\
Opens a file on the DVD, given the title number and domain.\n\n\
If the title number is 0, the video manager information is opened\n\
(VIDEO_TS.[IFO,BUP,VOB]).\n\n\
On success, returns a file structure which may be used for reads.\n\
On failure, raises a RuntimeError.") DVDOpenFile;
%exception DVDOpenFile {
    $action
    if (!result) {
        PyErr_SetString(PyExc_RuntimeError, "Unable to open file");
     return NULL;
  }
}
dvd_file_t *DVDOpenFile( dvd_reader_t *, int, dvd_read_domain_t );


/**
 * DVDCloseFile
 */
%feature("autodoc", "DVDCloseFile(dvd_file)\n\n\
Closes a file and frees the associated structure.") DVDCloseFile;
void DVDCloseFile( dvd_file_t * );


/**
 * DVDReadBlocks
 */
%cstring_output_withsize(unsigned char *data, size_t *size);
ssize_t _DVDReadBlocks(unsigned char *data, size_t *size, dvd_file_t *dvd, int offset);
%inline %{
ssize_t _DVDReadBlocks(unsigned char *data, size_t *size, dvd_file_t *dvd, int offset)
{
    ssize_t ret = DVDReadBlocks(dvd, offset, *size / DVD_VIDEO_LB_LEN, data);
    *size = (ret == -1)? 0: ret * DVD_VIDEO_LB_LEN;
    return ret;
}
%}
%pythoncode %{
def DVDReadBlocks(dvd_file, offset, block_count):
    """
    Reads block_count number of blocks from the file at the given block offset.
    
    On failure, raises RuntimeError.
    
    This call is only for reading VOB data, and should not be used when reading the IFO files.
    When reading from an encrypted drive, blocks are decrypted using libdvdcss where required.
    """
    ret, data = _DVDReadBlocks(block_count * DVD_VIDEO_LB_LEN, dvd_file, offset)
    if ret < 0:
        raise RuntimeError("Unable to read data")
    return data
%}


/**
 * DVDFileSeek
 */
%feature("autodoc", "DVDFileSeek(dvd_file, seek_offset) -> offset_set\n\n\
Seek to the given position in the file.\n\n\
Returns the resulting position in bytes from the beginning of the file.\n\n\
The seek position is only used for byte reads from the file,\n\
the block read call always reads from the given offset.") DVDFileSeek;
int DVDFileSeek( dvd_file_t *, int );


/**
 * DVDReadBytes
 */
%cstring_output_withsize(unsigned char *data, size_t *size);
ssize_t _DVDReadBytes(dvd_file_t *dvd_file, unsigned char *data, size_t *size);
%inline %{
ssize_t _DVDReadBytes(dvd_file_t *dvd_file, unsigned char *data, size_t *size)
{
    ssize_t ret = DVDReadBytes(dvd_file, data, *size);
    *size = (ret < 0)? 0: ret;
    return ret;
}
%}
%pythoncode %{
def DVDReadBytes(dvd_file, bytes):
    """
    Reads the given number of bytes from the file.
    
    This call can only be used on the information files,
    and may not be used for reading from a VOB.
    This reads from and increments the current seek position for the file.
    
    On failure, raises RuntimeError.
    """
    ret, data = _DVDReadBytes(dvd_file, bytes)
    if ret < 0:
        raise RuntimeError("Unable to read data")
    return data
%}


/**
 * DVDFileSize
 */
%feature("autodoc", "DVDFileSize(dvd_file) -> blocks\n\n\
Returns the file size in blocks.\n\n\
If an error occurs, RuntimeError is raised.") DVDFileSize;
%exception DVDFileSize {
    $action
    if (result == -1) {
        PyErr_SetString(PyExc_RuntimeError, "Couldn't get the file size");
     return NULL;
  }
}
ssize_t DVDFileSize( dvd_file_t * );


/**
 * DVDDiscID
 */
%rename(_DVDDiscID) DVDDiscID;
%cstring_chunk_output(unsigned char *discid, 16);
int DVDDiscID(dvd_reader_t *dvd, unsigned char *discid);
%pythoncode %{
def DVDDiscID(dvd):
    """
    Get a unique 128 bit disc ID.
    
    This is the MD5 sum of VIDEO_TS.IFO and the VTS_0?_0.IFO files
    in title order (those that exist).
    
    If you need a 'text' representation of the id, print it as a
    hexadecimal number, using lowercase letters, discid[0] first.
    I.e. the same format as the command-line 'md5sum' program uses.
    
    On success, returns a 16 char string.
    On error, raises a RuntimeError exception.
    """
    ret, id = _DVDDiscID(dvd)
    if ret == -1:
        raise RuntimeError("Couldn't read the disc ID")
    return id
%}


/**
 * DVDUDFVolumeInfo
 */
%rename(_DVDUDFVolumeInfo) DVDUDFVolumeInfo;
%cstring_bounded_output(char *volid, 32);
%cstring_chunk_output(unsigned char *volsetid, 128);
int DVDUDFVolumeInfo( dvd_reader_t *dvd, char *volid, unsigned int volid_size,
                      unsigned char *volsetid, unsigned int volsetid_size);
%pythoncode %{
def DVDUDFVolumeInfo(dvd):
    """
    Get the UDF VolumeIdentifier and VolumeSetIdentifier from the PrimaryVolumeDescriptor.
    
    On success, a tuple containing the VolumeIdentifier and VolumeSetIdentifier is returned.
    On failure, RuntimeError is raised.
    """
    ret, volid, volsetid = _DVDUDFVolumeInfo(dvd, 32, 128)
    if ret == -1:
        raise RuntimeError("Couldn't read the volume info.")
    return volid, volsetid
%}


/**
 * DVDISOVolumeInfo
 */
%rename(_DVDISOVolumeInfo) DVDISOVolumeInfo;
%cstring_bounded_output(char *volid, 33);
%cstring_chunk_output(unsigned char *volsetid, 128);
int DVDISOVolumeInfo( dvd_reader_t *dvd, char *volid, unsigned int volid_size,
                      unsigned char *volsetid, unsigned int volsetid_size);
%pythoncode %{
def DVDISOVolumeInfo(dvd):
    """
    Get the ISO9660 VolumeIdentifier and VolumeSetIdentifier
    
    * Only use this function as fallback if DVDUDFVolumeInfo fails       *
    * This will happen on a disc mastered only with a iso9660 filesystem *
    * All video DVD discs have UDF filesystem                            *
    
    On success, a tuple containing the VolumeIdentifier and VolumeSetIdentifier is returned.
    On failure, RuntimeError is raised.
    """
    ret, volid, volsetid = _DVDISOVolumeInfo(dvd, 33, 128)
    if ret == -1:
        raise RuntimeError("Couldn't read the volume info.")
    return volid, volsetid
%}


/**
 * DVDUDFCacheLevel
 */
%feature("autodoc", "DVDUDFCacheLevel(dvd, level) -> set_level\n\n\
Sets the level of caching that is done when reading from a device.\n\n\
The level parameter can take any of the following values:\n\
   -1 - returns the current setting.\n\
    0 - UDF Cache turned off.\n\
    1 - (default level) Pointers to IFO files and some data from\n\
        PrimaryVolumeDescriptor are cached.") DVDUDFCacheLevel;
int DVDUDFCacheLevel( dvd_reader_t *, int );

